/*
 * Ambient - the ultimate desktop
 * ------------------------------
 * Copyright 2006 Ambient Open Source Team
 * All Rights Reserved
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * $Id: sendrexx.c,v 1.2 2012/02/11 23:49:47 jacadcaps Exp $
 */

/* public */
#include <string.h>
#include <exec/execbase.h>
#include <rexx/errors.h>
#include <rexx/rxslib.h>
#include <rexx/storage.h>
#include <proto/exec.h>
#include <proto/rexxsyslib.h>

/* private */
#include "qport.h"
#include "sendrexx.h"

ULONG sendrexx(CONST_STRPTR cmd)
{
	struct Library *RexxSysBase;
	BOOL rc = FALSE;

	if ((RexxSysBase = OpenLibrary("rexxsyslib.library", 0)))
	{
		struct RexxMsg *rexxmsg;
		struct MsgPort rport;

		CreateQPort(&rport);

		if ((rexxmsg = CreateRexxMsg(&rport, NULL, "AMBIENT")))
		{
			rexxmsg->rm_Args[0] = CreateArgstring(cmd, strlen(cmd) + 1);

			if (rexxmsg->rm_Args[0])
			{
				struct MsgPort *amp;
				struct Task *caller = FindTask(NULL);

				rexxmsg->rm_Action = RXCOMM;

				Forbid();
				amp = (struct MsgPort *)FindExecNode(EXECLIST_PORT, "AMBIENT");

				// protect against this being called from the ambient thread!
				if (amp && amp->mp_SigTask != caller)
				{
					PutMsg(amp, &rexxmsg->rm_Node);
					Permit();
					WaitPort(&rport);
					(void)GetMsg(&rport);

					rc = rexxmsg->rm_Result1 == RC_OK ? TRUE : FALSE;
				}
				else
				{
					Permit();
				}

				DeleteArgstring(rexxmsg->rm_Args[0]);
			}

			DeleteRexxMsg(rexxmsg);
		}

		DeleteQPort(&rport);
		CloseLibrary(RexxSysBase);
	}

	return (rc);
}
